//------------------------------------------------------------------
//                                    EA_MACD_Divergence_v1-beta.mq4 
//                                  (c) Copyright 2006, Glyn Tomkins 
//                                            glyn_tomkins@yahoo.com 
//
//  PURPOSE:
//
//  This program is a MetaTrader V4 Expert Advisor which identifies
//  MACD Divergence to price and alerts the user via a variety of 
//  methods. 
//
//  LICENSE:
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
//  02110-1301, USA.
//
//  METHOD: 
//
//  1. First, init() sets up by going back thru the entire chart ... 
//  and make note so all the peaks and troughs.
//  
//  Note: A peak on MACD is /\ trough is \/ ... it doesn't look for 
//  troughs and peaks where the MACD indicator is equal on two 
//  adjacent bars (eg trough like \_/) ... now the chance of that 
//  happening is remote to say the least (unlike on price) so we are
//  usually OK with this indiscretion.  We might miss a possible setup
//  on an infrequent basis. 
//  
//  2. As each new bar forms, it looks to see if a new peak or trough
//  has formed. if one has, it goes to step 3 otherwise it waits for 
//  the next bar, i.e. repeats step 2. 
//  
//  note: the most current bar (bar 0) that is in progress is not 
//  used.
//  
//  3. Once a new trough (or peak) has formed ... it goes back and 
//  compares it to previous troughs (or peaks) .... if you can draw a 
//  direct line from the new trough (or peak) to the trough (or peak) 
//  being compared without it cutting the MACD Line itself AND the 
//  trough (peak) is lower (or higher) then it is compared the low (or
//  high) price at the trough (or peak) being compared to the newly 
//  formed trough (or peak) and if low (or high) price is dropping (or
//  rising) then divergence is declared.
//
//  POSSIBLE FUTURE EXPANSION: 
//
//  1. come up with a way to determine how far to go back ... it is 
//  feasible that this will alert back to a peak or trough way way 
//  back that is not terribly relevant. 
//
//  2. Convert into an Indicator that will add divergence lines to 
//  price and the MACD indicator. //
//
//  OPTIONS: 
//
//     MACDFast   - Period of Fast MA for MACD 
//     MACDSlow   - Period of Slow MA for MACD
//     MACDSignal - Period of MA of MACD (Trigger) 
//
//     email
//          true  - if you wish to receive emails. Note you have to 
//                  have enabled and configured this in MetaTrader
//          false - do not alert via email.
//
//     alerts
//          true  - if you wish to receive alerts through the 
//                  MetaTrader alert window. 
//          false - do not receive alerts on the Alert Window.
//
//     popups
//          true  - if you wish to receive alerts through popups 
//                  (like those you receive from Instant Messenger
//                  clients. 
//                  Note that you need to install the popup utility
//                  provided by codersguru at forex-tsd.com. It is 
//                  available for download at 
//                  http://www.metatrader.info/node/82
//          false - do not receive alerts as Popups.
//
//     popuploc     location of where you installed popup.dll. 
//
//     Debug
//          0 - off
//          1 - low logging of start only
//          2 - verbose logging of start only
//          3 - low logging of init and start
//          4 - verbose logging of init and start
//
//  CHANGE HISTORY: 
//
//     2006-03-29 Initial version
//------------------------------------------------------------------
#include <Pop.mqh>


#property copyright "(c) Copyright 2006, Glyn Tomkins"
#property link      "glyn_tomkins@yahoo.com"

//---- input parameters
extern int    MACDFast     = 12;
extern int    MACDSlow     = 26;
extern int    MACDSignal   = 9;

extern bool   email        = true;
extern bool   alerts       = true;
extern bool   popups       = true;
extern string popuploc     = "C:";
extern int    Debug        = 0;

double I[];
double T[];
double O[];
double H[];
double L[];
double C[];
int    IndicatorIndex[];
double IndicatorHeight[];
int    ThisBar;
double IRange;
int    SavedBars;


//+------------------------------------------------------------------+
//| function to strip trailing zeroes off prices                     |
//+------------------------------------------------------------------+

string PriceToStr(double n)
{

  switch (Point)
    {
    case 0.1: {return(DoubleToStr(n,1));break;}
    case 0.01: {return(DoubleToStr(n,2));break;}
    case 0.001: {return(DoubleToStr(n,3));break;}
    case 0.0001: {return(DoubleToStr(n,4));break;}
    case 0.00001: {return(DoubleToStr(n,5));break;}
    }
}
//+------------------------------------------------------------------+
//| string FormattedPeriod() function                                |
//+------------------------------------------------------------------+
string FormattedPeriod()
  {
    switch (Period())
    {
      case PERIOD_M1:
        return("1Min"); 
        break;     
      
      case PERIOD_M5: 
        return("5Mins"); 
        break;     

      case PERIOD_M15: 
        return("15Mins"); 
        break;     

      case PERIOD_M30: 
        return("30Mins"); 
        break;     

      case PERIOD_H1: 
        return("1Hr"); 
        break;     

      case PERIOD_H4: 
        return("4Hrs"); 
        break;     

      case PERIOD_D1: 
        return("1Day"); 
        break;     

      case PERIOD_W1:
        return("1Week"); 
        break;     

      case PERIOD_MN1: 
        return("1Month"); 
        break;   
    }      
  return ("");
  }
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+

int init()
  {
  
   Print("EA_MACD_Divergence Version 1 , Copyright (C) 2006, Glyn Tomkins");
   Print("EA_MACD_Divergence comes with ABSOLUTELY NO WARRANTY");
   Print("This is free software, and you are welcome to redistribute it");
   Print("under certain conditions; For details see license.txt.");
  
   //scan back thru MACD and index the peaks and troughs looking for the previous peak and trough and also capture the PRICE
   
   int    i,j;
   
   ArrayResize(I,Bars);                 // this is for storing the index - note that its indexed the oposite way to Metatrader
   ArrayResize(IndicatorIndex,Bars);    // this is for the length in bars of the peak/trough
   ArrayResize(IndicatorHeight,Bars);   // this is for the height of the peak/trough
   
   ArrayResize(O,Bars);        
   ArrayResize(H,Bars);
   ArrayResize(L,Bars);
   ArrayResize(C,Bars);
   ArrayResize(T,Bars);
   if (Debug >= 3)
     Print("Initiating. Bars: " + Bars + " From: " + TimeToStr(Time[Bars-1]) + " To: " + TimeToStr(CurTime()));  
  
  if (popups)
     pop("MACD divergence Enabled on "+Symbol() + " " + FormattedPeriod(),popuploc);     

   // grab the indicator and price and save
   // note the array is indexed in reverse for later expansion on each new bar

   for(ThisBar=Bars-1;ThisBar>=1; ThisBar--) // ignoring the bar in progress 
   {
     I[Bars - ThisBar] = /*DecimalRound(*/iMACD(NULL,0,MACDFast,MACDSlow,9,PRICE_CLOSE,MODE_MAIN,ThisBar);//,6);
     O[Bars - ThisBar] = Open[ThisBar];
     H[Bars - ThisBar] = High[ThisBar];
     L[Bars - ThisBar] = Low[ThisBar];
     C[Bars - ThisBar] = Close[ThisBar];
     T[Bars - ThisBar] = Time[ThisBar];  
 
     if ((Debug >= 4) || (Debug >= 3 && (ThisBar < 10 || ThisBar > Bars - 11)))
     {
       Print("Bar " + (Bars - ThisBar) + " T: " + TimeToStr(T[Bars - ThisBar]) + " O:"+O[Bars - ThisBar]+" H:"+H[Bars - ThisBar]+" L:"+L[Bars - ThisBar]+" C:"+C[Bars - ThisBar]+" I:"+I[Bars - ThisBar]);
     }
   }
   SavedBars = Bars;
//   SavedBars = Bars - 100; // this is to allow testing of the EA without waiting for the ticks/bars to form. 
   IRange = I[ArrayMaximum(I)] - I[ArrayMinimum(I)];  // save the indicator range
   
   if (Debug >= 3)
     {
      Print("Got the Indicator and price data");
     }
   // now go thru the indicator and find and mark the peaks and troughs 
   // this starts at the furthest time back and works forward 
   for(ThisBar = 2;ThisBar <= Bars - 2; ThisBar++) // start one in and end one short 
   {
      if ((I[ThisBar + 1] < I[ThisBar]) && (I[ThisBar] > I[ThisBar - 1]))
       {
        if (Debug >= 3)
         Print("Peak found at " + TimeToStr(T[ThisBar]) + " " + ThisBar + " " + I[ThisBar + 1] + " " + I[ThisBar] + " " + I[ThisBar - 1]);
         
       if ((ThisBar == 2) || (ThisBar == Bars - 2))
         IndicatorIndex[ThisBar] = 1;
       else  
       {
 
         for (i=ThisBar - 2, j=ThisBar + 2;((Bars-1>=j) && (i>=1));j++,i--)
         {
           IndicatorIndex[ThisBar] = ThisBar - i - 1;
           IndicatorHeight[ThisBar] = MathMax(I[ThisBar] - I[j - 1],I[ThisBar] - I[i + 1]);
  
           if ((I[j] < I[ThisBar]) && (I[ThisBar] > I[i]))
             continue;
           else
             break;
         }
         if (Debug >= 3)
           Print("Peak Length " + IndicatorIndex[ThisBar] + " Height " + IndicatorHeight[ThisBar]);
  
      }
     }
     else
     {
       if ((I[ThisBar + 1] > I[ThisBar]) && (I[ThisBar] < I[ThisBar - 1]))
       {
         // we have a trough - now just find out how big it is
         if (Debug >= 3)
           Print("trough found at " + TimeToStr(T[ThisBar])+ " " + ThisBar + " " + I[ThisBar + 1] + " " + I[ThisBar] + " " + I[ThisBar - 1]);
 
         if ((ThisBar == 2) || (ThisBar == Bars - 2))
         {
           IndicatorIndex[ThisBar] = -1;
         }
         else  
         {
           // set up the defaults for these values in case we drop out of the loop
 
           for (i=ThisBar - 2, j=ThisBar + 2;((Bars-1 >= j) && (i>=1));j++,i--)
           {
             IndicatorIndex[ThisBar] = ThisBar - j + 1;
             IndicatorHeight[ThisBar] = MathMax(I[j - 1] - I[ThisBar],I[i + 1] - I[ThisBar]);
 
             if ((I[j] > I[ThisBar]) && (I[ThisBar] < I[i]))
               continue;
             else
               break;
           }
           if (Debug >= 3)
             Print("Trough Length " + IndicatorIndex[ThisBar] + " Height " + IndicatorHeight[ThisBar]);
         }
       }
       else
       {
         // neither peak nor trough
         IndicatorIndex[ThisBar] = 0;
        if (Debug >= 4)
           Print("Nothing found at " + TimeToStr(T[ThisBar]) + I[ThisBar + 1] + I[ThisBar] + I[ThisBar - 1]);
       }
     }
   }  

   if (Debug >=3)
   {
     for(ThisBar=Bars-1;ThisBar>=Bars-100;ThisBar--)
     {
       if (IndicatorIndex[ThisBar] > 0)
         Print("Peak: strength " + IndicatorIndex[ThisBar] + " Height " + MathRound(100 * IndicatorHeight[ThisBar]/IRange) + "% Bar " + ThisBar + " at " + TimeToStr(T[ThisBar]));
       else
         if (IndicatorIndex[ThisBar] < 0)
           Print("Trough: strength " + IndicatorIndex[ThisBar] + " Height " + MathRound(100 * IndicatorHeight[ThisBar]/IRange) + "% Bar " + ThisBar + " at " + TimeToStr(T[ThisBar]));
         else
           if (IndicatorIndex[ThisBar] == 0)
             Print("Nothing: " + IndicatorIndex[ThisBar] + " Bar " + ThisBar + " at " + TimeToStr(T[ThisBar]));
 
     }
   }
 
   return(0);
  }  // end of init()
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
if (popups)
     pop("MACD divergence Disabled on "+Symbol() + " " + FormattedPeriod(),popuploc);     

//----
   return(0);
  }

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{ 
  double TangentofAngleofSight = 0;
  int Divergence;
  string EmailText, EmailSubject;
  bool peak,trough; 

  if (Bars > SavedBars)
  {
    // we have a another bar
    // make the array bigger
    ArrayResize(I,Bars);                 
    ArrayResize(IndicatorIndex,Bars);   
    ArrayResize(IndicatorHeight,Bars); 
 
    ArrayResize(O,Bars); 
    ArrayResize(H,Bars);
    ArrayResize(L,Bars);
    ArrayResize(C,Bars);
    ArrayResize(T,Bars);
    
    // save the values
    I[Bars-1] = iMACD(NULL,0,MACDFast,MACDSlow,9,PRICE_CLOSE,MODE_MAIN,1);
    O[Bars-1] = Open[1];
    H[Bars-1] = High[1];
    L[Bars-1] = Low[1];
    C[Bars-1] = Close[1];
    T[Bars-1] = Time[1]; 
    SavedBars = Bars; 
//    SavedBars ++; // use this for testing ... comment out previous line and uncomment this one. 
    IRange = I[ArrayMaximum(I)] - I[ArrayMinimum(I)];  // save the indicator range
  
    if (Debug>=2)
    {
      Print("New bar at " + TimeToStr(T[SavedBars-1])+ " " + (SavedBars-1) + " O:"+O[SavedBars-1]+" H:"+H[SavedBars-1]+" L:"+L[SavedBars-1]+" C:"+C[SavedBars-1]+" I:"+I[SavedBars-1]);
    }  
    // ok .. now see if a new peak or trough has occured .. if not do nothing more
  
    peak    = false;
    trough  = false;
 
    if ((I[SavedBars-1] > I[SavedBars-2]) || (I[SavedBars-1] < I[SavedBars-2]))
    {
      if (Debug >= 1)
        if ((I[SavedBars-1] > I[SavedBars-2]))
          Print("MACD up to " + I[SavedBars-1]);
        else
          Print("MACD down to " + I[SavedBars-1]);
 
      // note that if these two are the same then we'll pick them up with the next bar that forms
      // peak of trough maybe forming
      ThisBar = SavedBars - 3;
      while (I[ThisBar] == I[ThisBar+1]  && ThisBar >= 1)  // move back in time thru period until indicator changes
      {
         ThisBar--;
         Print("Stepping back to " + TimeToStr(T[ThisBar]));
      }  
     
      if ((I[SavedBars-1] > I[SavedBars-2]) && (I[ThisBar+1] < I[ThisBar]))
      {
        // new trough
        trough = true;
        if (Debug>=1)
          Print("New trough detected");
      }
      else
      {
        if ((I[SavedBars-1] < I[SavedBars-2])&&(I[ThisBar+1] > I[ThisBar]))
        {
          // new peak
          peak = true;
          if (Debug>=1)
            Print("New peak detected");
        }
      }
    }
    if (peak || trough)
    {
      // OK - we have something to do here - peak or trough at index bar-2
      Divergence = 0;
      EmailText = "";
      TangentofAngleofSight = 0;      
      for (ThisBar = ThisBar-1;ThisBar > 0;ThisBar--)
      {
        if (Debug >= 2 && trough && (IndicatorIndex[ThisBar] < 0))
        {
          Print("Comparing with trough at " + TimeToStr(T[ThisBar]) + " I(Then): " + I[ThisBar] + " I(Now): " + I[SavedBars - 2] + " L(Then): " + L[ThisBar] + " L(Now): " + L[SavedBars - 2]) ;
          Print(I[SavedBars-2] + " " + I[ThisBar] + " " +  IRange  + " " +  SavedBars  + " " +  ThisBar);
          Print((100*(I[ThisBar]-I[SavedBars-2])/IRange)/(SavedBars - ThisBar));
  
        }

        // loop thru the saved data looking for previous higher peak/ lower trough searching the most recent First
        if (trough && (IndicatorIndex[ThisBar] < 0) && (I[ThisBar] <= I[SavedBars-2]) && TangentofAngleofSight >= (100*(I[ThisBar]-I[SavedBars-2])/IRange)/(SavedBars - 2 - ThisBar))
        {
          // we were looking for a trough and we found one and its lower than the one just formed and it is in direct line of sight
 
          // so first save the angle of sight
          if (Debug >= 2)
          {
            Print("Checking for divergence") ;
          }
          TangentofAngleofSight = 100*((I[ThisBar]-I[SavedBars-2])/IRange)/(SavedBars-2 - ThisBar);
          
          // Now test for divergence at this pair
          if (L[ThisBar] > L[SavedBars-2])
          {
            // i am calling this +ve divergence
            Divergence++;
            EmailText = EmailText + "+ve divergence:\tTime:"+TimeToStr(T[ThisBar])+"\t Indicator:"+I[ThisBar]+"\t Open:"+PriceToStr(O[ThisBar])+"\t High:"+PriceToStr(H[ThisBar])+"\t Low:"+PriceToStr(L[ThisBar])+"\t Close:"+PriceToStr(C[ThisBar])+"\n";
          }
          else
          {
            if (Debug >= 1)
              Print("No +ve divergence detected");
          }
        }
        else
        {
          if (Debug >= 2 && peak  && (IndicatorIndex[ThisBar] > 0))
          {
            Print("Comparing with peak  at " + TimeToStr(T[ThisBar]) + " I(Then): " + I[ThisBar] + " I(Now): " + I[SavedBars - 2] + " H(Then): " + H[ThisBar] + " H(Now): " + H[SavedBars - 2]) ;
            Print(I[SavedBars-2] + " " + I[ThisBar] + " " +  IRange  + " " +  SavedBars  + " " +  ThisBar);
            Print(100*((I[ThisBar]-I[SavedBars-2])/IRange)/(SavedBars - ThisBar));
          }
          if (peak && (IndicatorIndex[ThisBar] > 0) && (I[ThisBar] >= I[SavedBars-2]) && TangentofAngleofSight <= (100*(I[ThisBar]-I[SavedBars-2])/IRange)/(SavedBars - 2 - ThisBar))
          {
            // we were looking for a peak AND we found one AND its higher than the one just formed AND it is in direct line of sight
   
            // so first save the angle of sight
            if (Debug >= 2)
            {
              Print("Checking for divergence");
            }
            TangentofAngleofSight = 100*((I[ThisBar]-I[SavedBars-2])/IRange)/(SavedBars-2 - ThisBar);
            
            // Now test for divergence at this pair
            if (H[ThisBar] < H[SavedBars-2])
            {
              // i am calling this -ve divergence
              Divergence++;
              EmailText = EmailText + "-ve divergence:\tTime:"+TimeToStr(T[ThisBar])+"\t Indicator:"+I[ThisBar]+"\t Open:"+PriceToStr(O[ThisBar])+"\t High:"+PriceToStr(H[ThisBar])+"\t Low:"+PriceToStr(L[ThisBar])+"\t Close:"+PriceToStr(C[ThisBar])+"\n";
            } // of +ve divergence
            else
            {
              if (Debug >= 1)
                Print("No -ve divergence detected");
            }
          } // of finding a peak of significance
        } // of not a trough
      } // of loop back

      // ok - did we find divergence?
      if (Divergence > 0)
      {
    if (peak)
        {
          EmailSubject = Divergence + " Negative divergence on "+Symbol() + " " + FormattedPeriod() + " at " + TimeToStr(T[SavedBars - 2]);     
          EmailText = "-ve divergence:\tTime:"+TimeToStr(T[SavedBars - 2])+"\t Indicator:"+I[SavedBars-2]+"\t Open:"+PriceToStr(O[SavedBars-2])+"\t High:"+PriceToStr(H[SavedBars-2])+"\t Low:"+PriceToStr(L[SavedBars-2])+"\t Close:"+ PriceToStr(C[SavedBars-2])+"\n\n" + EmailText;
        }
        else // must be a trough
        {
          EmailSubject = Divergence + " Positive divergence on "+Symbol() + " " + FormattedPeriod() + " at " + TimeToStr(T[SavedBars - 2]);
          EmailText = "+ve divergence:\tTime:"+TimeToStr(T[SavedBars - 2])+"\t Indicator:"+I[SavedBars-2]+"\t Open:"+PriceToStr(O[SavedBars-2])+"\t High:"+PriceToStr(H[SavedBars-2])+"\t Low:"+PriceToStr(L[SavedBars-2])+"\t Close:"+PriceToStr(C[SavedBars-2])+"\n\n" + EmailText;
        }
        if (email)
          SendMail(EmailSubject,EmailText);
          
        if (popups)
          pop(EmailSubject,popuploc);     
 
        if (alerts)
          Alert(EmailSubject);
 
        if (Debug>=1)
        {
          Print(EmailSubject);
          Print(EmailText);
        }   
      } // of divergence
    } // of finding a peak   
  } // of the new bar forming
  else
  {
    // no new bar ... just sit and wait
  }
    
   return(0);
  }
//+------------------------------------------------------------------+